﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Problems.BinPacking.Interfaces;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;

namespace HeuristicLab.Problems.BinPacking.Shapes {
  [Item("PackingShape", "Represents an abstract shape to describe the measures and the positioning of objects related to bin-packing problems.")]
  [StorableClass]
  public abstract class PackingShape<D> : NamedItem, IPackingShape
    where D: class, IPackingDimensions
  {
    public static Type PositionType {
      get { return typeof(D); }
    }

    public abstract bool Encloses(D checkedPosition, PackingShape<D> checkedShape);
    public abstract bool Overlaps(D myPosition, D checkedPosition, PackingShape<D> checkedShape);
    public abstract void InitializeFromMeasures(int[] measures);
    public abstract int[] ToArray();

    public PackingShape(int[] measures) {
      InitializeFromMeasures(measures);
    }
    public PackingShape() {

    }


    [StorableConstructor]
    protected PackingShape(bool deserializing) : base(deserializing) { }
    protected PackingShape(PackingShape<D> original, Cloner cloner)
      : base(original, cloner) {
    }
  }
}
